Class.create(function(Super)
 /**
  * StackTraceNX
  * @param Super [Function|null] X[p[NX
  * @return Function js.debug.jsem.trace.StackTraceNX
  * 
  * @package js.debug.jsem.trace
  * @update 2007/06/16
  * @version 2.00.00
  * @author Mug
  * @copyright (C) 2006-2007 Mug, All rights reserved.
  * 
  * @explanation <p>̃NX́AX^bNg[Xi[ĂACX^XɏWB
  *   <var>js.debug.jsem.Exception</var>܂͂̃TuNX̃CX^X𐶐ƂA
  *   ̃NX̃CX^XB</p>
  * @rel js.debug.jsem.Exception
  * @example <p><var>js.debug.jsem.Exception</var><var>js.cpt.Error</var>琶A
  *   X^bNg[X<var>js.cpt.Log</var>ɏo͂B</p>
  *   <pre><code>
  *   var log = new js.cpt.Log("Log sample of StackTrace");
  *   
  *   function a(arg) {
  *     if(arg instanceof Array) {
  *       return arg.length;
  *     } else {
  *       throw new Error("Exception", "argArrayIuWFNgł͂܂B");
  *     }
  *   }
  *   
  *   function b(num) {
  *     var list = (num > 0) ? [] : null;
  *     return num + a(list);
  *   }
  *   
  *   function main() {
  *     for(var i = 3; i >= -3; i--) {
  *       log.info("[", i, "]=", b(i));
  *     }
  *   }
  *   
  *   try {
  *     main();
  *   } catch(e) {
  *     log.error("X^bNg[X");
  *     log.error(e.stackTrace);
  *   }
  *   </code></pre>
  *   <p>̃R[hsƂ̃Óic[ɂo͂镔jʂ͈ȉ̂ƂB</p>
  *   <pre><code>
  *   [3]=3
  *   [2]=2
  *   [1]=1
  *   X^bNg[X
  *   function Class(cause, message)
  *     arguments=[null, "argArrayIuWFNgł͂܂B"]
  *   function Class(name, cause, message)
  *     arguments=["Exception", null]
  *   function a(arg)
  *     arguments=[null]
  *   function b(num)
  *     arguments=[0]
  *   function main()
  *     arguments=[]
  *   </code></pre>
  *   <p>̏o͌ʂ̓ftHgł邪Aw肵toString\bhsA
  *   e<var>js.debug.jsem.trace.TraceData</var>IuWFNgɑ΂ĔCӂ̕񉻏sA
  *   X^bNg[XƂďo͂悤ȃR[hLq邱Ƃ\łB</p>
  * @rel js.debug.jsem.trace.TraceData
  */
  {
  //##############################################################################
  //
  // wb_(vpeBEpENXpX)
  //
  //
  // vpeB - NXɊւ郁^f[^
  //
  //:V Creator Function NXNG[^[
  var Creator = arguments.callee;
  // SCNX
  Creator.CompleteName = "js.debug.jsem.trace.StackTrace";
  
  //
  // p - vg^CvpƊ{õZbg
  //
  // p
  Creator.inherit(Class, Super);
  //:V Static Object StaticIuWFNg
  var Static = Class.Static;
  //:V Proto Object vg^CvIuWFNg
  var Proto = Class.prototype;
  
  //
  // NXpX
  //
  // js.debug.jsem.trace.TraceData̎Q
  var TraceData = js.debug.jsem.trace.TraceData;
  
  //##############################################################################
  //
  // {fB(vCx[gENXEStaticEvg^CvE)
  //
  //
  // vCx[g - NX̔Joƃ\bh(pȂ)
  //
  //------------------------------------------------------------------------------
  function checkListContains(list, obj)
   /**
    * Xg̗vfɑΏۂ܂ł邩ǂ
    * @param list Array Xg
    * @param obj Variant ܂܂Ă邩ǂΏ
    * @return Boolean ܂܂Ă邩ǂ
    */
    {
    var contains = false;
    var i = list.length;
    while(i > 0)
      {
      if(list[--i] == obj)
        {
        contains = true;
        break;
        }
      }
    return contains;
    }
  
  //------------------------------------------------------------------------------
  function collectTraces(baseFunc)
   /**
    * X^bNg[XW
    * @param baseFunc Function WƂȂ֐i̊֐WΏہj
    * @return Array js.debug.jsem.trace.TraceDataCX^X̔zinullj
    */
    {
    var func = baseFunc;
    var args = func.arguments;
    var dataList = [];
    var funcList = [];
    var argsList = [];
    while((typeof func) == "function")
      {
      dataList[dataList.length] = new TraceData(func, args);
      funcList[funcList.length] = func;
      argsList[argsList.length] = args;
      // next
      func = (func.caller && !checkListContains(funcList, func.caller)) ? func.caller : null;
      args = (args.caller && !checkListContains(argsList, args.caller)) ? args.caller : null;
      if(!func && !args)
        {
        break;
        }
      else if(func && !args)
        {
        args = func.arguments;
        }
      else if(!func && args)
        {
        func = args.callee;
        }
      else if(func != args.callee)
        {
        func = args.callee;
        }
      }
    delete funcList;
    delete argsList;
    return dataList;
    }
  
  //
  // NX - NX̌Joƃ\bh(pȂ)
  //
  
  //
  // Static - NX.StatičJoƃ\bh(p)
  //
  
  //
  // vg^Cv - NX.prototypěJoƃ\bh(p)
  //
  //------------------------------------------------------------------------------
  Proto.toString = function(lineSeparator)
   /**
    * X^bNg[X̕\(ftHg)擾
    * @explanation PsɕtP̃g[Xf[^̕\ĕԂB
    * @param lineSeparator [String] s(ȗ:\n)
    * @return String X^bNg[X̕\(ftHg)
    */
    {
    if((typeof lineSeparator) != "string")
      {
      lineSeparator = "\n";
      }
    var str = "";
    var size = this.getSize();
    for(var i = 0; i < size; i++)
      {
      str += this.getTraceData(i).toString(lineSeparator) + lineSeparator;
      }
    return str;
    };
  
  //
  //  - NX`ɂP񂾂s
  //
  
  //##############################################################################
  //
  // RXgN^(vCx[gEthisERXgN^)
  //
  function Class(func)
   /**
    * StackTraceNX̃RXgN^
    * @param func [Function] X^bNg[XWƂȂ֐
    * @return js.debug.jsem.trace.StackTrace CX^X
    */
    {
    //
    // X[p[NXKp(p) - X[p[NX̃RXgN^N
    //
    //Super.apply(this, arguments);
    //
    // vCx[g - CX^X̔Joƃ\bh(pȂ)
    //
    //:v trace Array g[Xf[^Xg(js.debug.jsem.trace.TraceData̔z)
    var trace = collectTraces(func);
    
    //
    // this - CX^X̌Joƃ\bh(Kpɂp)
    //
    //------------------------------------------------------------------------------
    this.getSize = function()
     /**
      * g[Xf[^Xg̃TCY擾
      * @return Number g[Xf[^Xg̃TCY
      */
      {
      return trace.length;
      };
    //------------------------------------------------------------------------------
    this.getTraceData = function(index)
     /**
      * g[Xf[^擾
      * @param index Number g[Xf[^Xg̃CfbNX
      * @return js.debug.jsem.trace.TraceData|null g[Xf[^
      */
      {
      var traceData = null;
      if(((typeof index) == "number") && (0 <= index) && (index < trace.length) && ((index % 1) == 0))
        {
        traceData = trace[index];
        if(!traceData)
          {
          traceData = null;
          }
        }
      return traceData;
      };
    
    //
    // RXgN^ - CX^X̌Joƃ\bh(pȂ)
    //
    
    return this;
    }
  
  return Class;
  });

